#!/usr/local/bin/scsh \
-dm -o program -e main -s
!#
     
;;; spmr.scm --- Scheme procmail replacement

;;; Copyright (C) 2001 by Walter C. Pelissero

;;; Author: Walter C. Pelissero <walter@pelissero.org>
;;; $Id: spmr.scm,v 2.13 2003/09/24 11:11:55 wcp Exp $

;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 2, or
;;; (at your option) any later version.
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;; You should have received a copy of the GNU General Public License
;;; along with this program; see the file COPYING.  If not, write to
;;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;;; Boston, MA 02111-1307, USA.

;;; Commentary:

(define-interface utils-interface
  (export ((define-lazy lazy when push w/cc within-temp-directory
	     with-open-file with-exclapp thunk) :syntax)
	  curry modify-cdr! call/cc member-if string-empty? negate
	  call-with-open-file remove-leading-whitespace
	  remove-trailing-whitespace string-capitalize bior
	  trim-whitespace make-temp-dir call-within-temp-directory
	  file->strings file->string))

(define-structure utils
  utils-interface
  (open scheme scsh string-lib list-lib)
  (files utils))

(define-interface user-interface
  (export
   ((header h~ when) :syntax)
   *reply-to-in-replies* *from-in-replies* mailbox user
   forward answer answer-mime save print-message
   printable-message quoted-message stop
   mime-message? for-each-mime-part map-mime-parts
   blacklist blacklisted? forward-to-program))

(define-interface internals-interface
  (export header parse-message with-mime-parts
	  sendmail make-mime-message run-rules))

(define-structure internals
  (compound-interface user-interface internals-interface)
  (open scheme scsh string-lib fluids
	package-commands-internal
	utils)
  (files internals))

(define-structure user-utils
  (compound-interface utils-interface user-interface)
  (open scheme scsh fluids
	utils internals)
  (begin))

;;; This is the environment for the user code to run in
(define-structure user-environment
  (export)
  (open scheme scsh fluids
	user-utils)
  (begin))

(define-structure program
  (export main)
  (open scheme scsh string-lib fluids
	internals
	;; user-environment is empty but we need to specify it here
	;; otherwise we get asked to load it at runtime
	user-environment)
  (begin
    (define (main args)
      ;; (set! home-directory (user-info:home-dir (user-info (user-uid))))
      (set-fluid! exec-path-list '("/bin" "/usr/bin" "/usr/sbin" "/usr/local/bin"))
      (if (null? (cdr args))
	  (parse-message (current-input-port))
	  (call-with-input-file (cadr args) parse-message))
      (run-rules 'user-environment)
      ;; By default we save the message in the usual mailbox.  We may
      ;; not reach this statement if rules contain an exit statement.
      (save)
      (exit))))



;;;
;;; Leave the following lines at end of file.
;;;
;;; Local Variables:
;;; mode:scheme
;;; scheme-program-name:"scsh"
;;; End:
